<?php

namespace Litecms\Content\Http\Controllers;

use App\Http\Controllers\ResourceController as BaseController;
use Form;
use Litecms\Content\Http\Requests\ContentRequest;
use Litecms\Content\Interfaces\ContentRepositoryInterface;
use Litecms\Content\Models\Content;

/**
 * Resource controller class for content.
 */
class ContentResourceController extends BaseController
{

    /**
     * Initialize content resource controller.
     *
     * @param type ContentRepositoryInterface $content
     *
     * @return null
     */
    public function __construct(ContentRepositoryInterface $content)
    {
        parent::__construct();
        $this->repository = $content;
        $this->repository
            ->pushCriteria(\Litepie\Repository\Criteria\RequestCriteria::class)
            ->pushCriteria(\Litecms\Content\Repositories\Criteria\ContentResourceCriteria::class);
    }

    /**
     * Display a list of content.
     *
     * @return Response
     */
    public function index(ContentRequest $request)
    {
        $view = $this->response->theme->listView();

        if ($this->response->typeIs('json')) {
            $function = camel_case('get-' . $view);
            return $this->repository
                ->setPresenter(\Litecms\Content\Repositories\Presenter\ContentPresenter::class)
                ->$function();
        }

        $contents = $this->repository->paginate();

        return $this->response->title(trans('content::content.names'))
            ->view('content::content.index', true)
            ->data(compact('contents'))
            ->output();
    }

    /**
     * Display content.
     *
     * @param Request $request
     * @param Model   $content
     *
     * @return Response
     */
    public function show(ContentRequest $request, Content $content)
    {

        if ($content->exists) {
            $view = 'content::content.show';
        } else {
            $view = 'content::content.new';
        }

        return $this->response->title(trans('app.view') . ' ' . trans('content::content.name'))
            ->data(compact('content'))
            ->view($view, true)
            ->output();
    }

    /**
     * Show the form for creating a new content.
     *
     * @param Request $request
     *
     * @return Response
     */
    public function create(ContentRequest $request)
    {

        $content = $this->repository->newInstance([]);
        return $this->response->title(trans('app.new') . ' ' . trans('content::content.name')) 
            ->view('content::content.create', true) 
            ->data(compact('content'))
            ->output();
    }

    /**
     * Create new content.
     *
     * @param Request $request
     *
     * @return Response
     */
    public function store(ContentRequest $request)
    {
        try {
            $attributes              = $request->all();
            $attributes['user_id']   = user_id();
            $attributes['user_type'] = user_type();
            $content                 = $this->repository->create($attributes);

            return $this->response->message(trans('messages.success.created', ['Module' => trans('content::content.name')]))
                ->code(204)
                ->status('success')
                ->url(guard_url('content/content/' . $content->getRouteKey()))
                ->redirect();
        } catch (Exception $e) {
            return $this->response->message($e->getMessage())
                ->code(400)
                ->status('error')
                ->url(guard_url('/content/content'))
                ->redirect();
        }

    }

    /**
     * Show content for editing.
     *
     * @param Request $request
     * @param Model   $content
     *
     * @return Response
     */
    public function edit(ContentRequest $request, Content $content)
    {
        return $this->response->title(trans('app.edit') . ' ' . trans('content::content.name'))
            ->view('content::content.edit', true)
            ->data(compact('content'))
            ->output();
    }

    /**
     * Update the content.
     *
     * @param Request $request
     * @param Model   $content
     *
     * @return Response
     */
    public function update(ContentRequest $request, Content $content)
    {
        try {
            $attributes = $request->all();

            $content->update($attributes);
            return $this->response->message(trans('messages.success.updated', ['Module' => trans('content::content.name')]))
                ->code(204)
                ->status('success')
                ->url(guard_url('content/content/' . $content->getRouteKey()))
                ->redirect();
        } catch (Exception $e) {
            return $this->response->message($e->getMessage())
                ->code(400)
                ->status('error')
                ->url(guard_url('content/content/' . $content->getRouteKey()))
                ->redirect();
        }

    }

    /**
     * Remove the content.
     *
     * @param Model   $content
     *
     * @return Response
     */
    public function destroy(ContentRequest $request, Content $content)
    {
        try {

            $content->delete();
            return $this->response->message(trans('messages.success.deleted', ['Module' => trans('content::content.name')]))
                ->code(202)
                ->status('success')
                ->url(guard_url('content/content/0'))
                ->redirect();

        } catch (Exception $e) {

            return $this->response->message($e->getMessage())
                ->code(400)
                ->status('error')
                ->url(guard_url('content/content/' . $content->getRouteKey()))
                ->redirect();
        }

    }

    /**
     * Remove multiple content.
     *
     * @param Model   $content
     *
     * @return Response
     */
    public function delete(ContentRequest $request, $type)
    {
        try {
            $ids = hashids_decode($request->input('ids'));

            if ($type == 'purge') {
                $this->repository->purge($ids);
            } else {
                $this->repository->delete($ids);
            }

            return $this->response->message(trans('messages.success.deleted', ['Module' => trans('content::content.name')]))
                ->status("success")
                ->code(202)
                ->url(guard_url('content/content'))
                ->redirect();

        } catch (Exception $e) {

            return $this->response->message($e->getMessage())
                ->status("error")
                ->code(400)
                ->url(guard_url('/content/content'))
                ->redirect();
        }

    }

    /**
     * Restore deleted contents.
     *
     * @param Model   $content
     *
     * @return Response
     */
    public function restore(ContentRequest $request)
    {
        try {
            $ids = hashids_decode($request->input('ids'));
            $this->repository->restore($ids);

            return $this->response->message(trans('messages.success.restore', ['Module' => trans('content::content.name')]))
                ->status("success")
                ->code(202)
                ->url(guard_url('/content/content'))
                ->redirect();

        } catch (Exception $e) {

            return $this->response->message($e->getMessage())
                ->status("error")
                ->code(400)
                ->url(guard_url('/content/content/'))
                ->redirect();
        }

    }

}
